CROWNSTONE_PLUG_ADVERTISEMENT_SERVICE_UUID = "C001"
CROWNSTONE_BUILTIN_ADVERTISEMENT_SERVICE_UUID = "C002"
CROWNSTONE_GUIDESTONE_ADVERTISEMENT_SERVICE_UUID  = "C003"
DFU_ADVERTISEMENT_SERVICE_UUID = "00001530-1212-efde-1523-785feabcd123"

class CSServices:
	DeviceInformation         = "0000180a-0000-1000-8000-00805f9b34fb"
	CrownstoneService         = "24f00000-7d10-4805-bfc1-7663a01c3bff"
	SetupService              = "24f10000-7d10-4805-bfc1-7663a01c3bff"

class DFUServices:
	DFU = DFU_ADVERTISEMENT_SERVICE_UUID

